@@ -45,6 +45,9 @@
 
 G_DEFINE_ABSTRACT_TYPE (BaobabChart, baobab_chart, GTK_TYPE_WIDGET);
 
+#define BAOBAB_CHART_MAX_DEPTH 8
+#define BAOBAB_CHART_MIN_DEPTH 1
+
 enum
 {
   LEFT_BUTTON   = 1,
@@ -52,7 +55,6 @@ enum
   RIGHT_BUTTON  = 3
 };
 
-
 struct _BaobabChartPrivate
 {
   guint name_column;
@@ -116,9 +118,8 @@ static void baobab_chart_get_property (GObject *object,
                                        GValue *value,
                                        GParamSpec *pspec);
 static void baobab_chart_free_items (GtkWidget *chart);
-static void baobab_chart_draw (GtkWidget *chart,
-                               cairo_t *cr,
-                               GdkRectangle area);
+static void baobab_chart_draw_chart (GtkWidget *chart,
+                                     cairo_t *cr);
 static void baobab_chart_update_draw (BaobabChart *chart,
                                       GtkTreePath *path);
 static void baobab_chart_row_changed (GtkTreeModel *model,
@@ -141,8 +142,8 @@ static void baobab_chart_rows_reordered (GtkTreeModel *model,
                                          GtkTreeIter *iter,
                                          gint *new_order,
                                          gpointer data);
-static gboolean baobab_chart_expose (GtkWidget *chart,
-                                     GdkEventExpose *event);
+static gboolean baobab_chart_draw (GtkWidget *chart,
+                                   cairo_t *cr);
 static void baobab_chart_interpolate_colors (BaobabChartColor *color,
                                              BaobabChartColor colora,
                                              BaobabChartColor colorb,
@@ -184,7 +185,7 @@ baobab_chart_class_init (BaobabChartClass *class)
 
   /* GtkWidget signals */
   widget_class->realize = baobab_chart_realize;
-  widget_class->expose_event = baobab_chart_expose;
+  widget_class->draw = baobab_chart_draw;
   widget_class->size_allocate = baobab_chart_size_allocate;
   widget_class->scroll_event = baobab_chart_scroll;
 
@@ -195,6 +196,8 @@ baobab_chart_class_init (BaobabChartClass *class)
   class->calculate_item_geometry = NULL;
   class->is_point_over_item      = NULL;
   class->get_item_rectangle      = NULL;
+  class->can_zoom_in             = NULL;
+  class->can_zoom_out            = NULL;
 
   g_object_class_install_property (obj_class,
                                    PROP_MAX_DEPTH,
@@ -295,31 +298,37 @@ baobab_chart_realize (GtkWidget *widget)
   BaobabChart *chart;
   GdkWindowAttr attributes;
   gint attributes_mask;
+  GtkAllocation allocation;
+  GdkWindow *window;
 
   g_return_if_fail (BAOBAB_IS_CHART (widget));
 
   chart = BAOBAB_CHART (widget);
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
+
+  gtk_widget_get_allocation (widget, &allocation);
 
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.x = widget->allocation.x;
-  attributes.y = widget->allocation.y;
-  attributes.width = widget->allocation.width;
-  attributes.height = widget->allocation.height;
+  attributes.x = allocation.x;
+  attributes.y = allocation.y;
+  attributes.width = allocation.width;
+  attributes.height = allocation.height;
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.visual = gtk_widget_get_visual (widget);
-  attributes.colormap = gtk_widget_get_colormap (widget);
   attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
 
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
-  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-                                   &attributes,
-                                   attributes_mask);
-  gdk_window_set_user_data (widget->window, chart);
+  window = gdk_window_new (gtk_widget_get_parent_window (widget),
+                           &attributes,
+                           attributes_mask);
+  gtk_widget_set_window (widget, window);
+  gdk_window_set_user_data (window, chart);
 
-  widget->style = gtk_style_attach (widget->style, widget->window);
-  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+  gtk_widget_style_attach (widget);
+  gtk_style_set_background (gtk_widget_get_style (widget),
+                            window,
+                            GTK_STATE_NORMAL);
 
   gtk_widget_add_events (widget,
                          GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK |
@@ -342,11 +351,11 @@ baobab_chart_size_allocate (GtkWidget *widget,
   priv = BAOBAB_CHART (widget)->priv;
   class = BAOBAB_CHART_GET_CLASS (widget);
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     {
-      gdk_window_move_resize (widget->window,
+      gdk_window_move_resize (gtk_widget_get_window (widget),
                   allocation->x, allocation->y,
                   allocation->width, allocation->height);
 
@@ -495,7 +504,7 @@ static void
 baobab_chart_get_items (GtkWidget *chart, GtkTreePath *root)
 {
   BaobabChartPrivate *priv;
-  BaobabChartItem *item, *child_item;
+  BaobabChartItem *item;
 
   GList *node;
   GtkTreeIter initial_iter = {0};
@@ -508,7 +517,6 @@ baobab_chart_get_items (GtkWidget *chart, GtkTreePath *root)
   GList *child_node;
   BaobabChartItem *child;
   gdouble rel_start;
-  gdouble rel_size;
 
   priv = BAOBAB_CHART_GET_PRIVATE (chart);
 
@@ -516,7 +524,12 @@ baobab_chart_get_items (GtkWidget *chart, GtkTreePath *root)
   baobab_chart_free_items (chart);
 
   /* Get the tree iteration corresponding to root */
-  gtk_tree_model_get_iter (priv->model, &initial_iter, root);
+  if (!gtk_tree_model_get_iter (priv->model, &initial_iter, root))
+    {
+      priv->model_changed = FALSE;
+      return;
+    }
+
 
 
   model_root_path = gtk_tree_path_new_first ();
@@ -549,7 +562,7 @@ baobab_chart_get_items (GtkWidget *chart, GtkTreePath *root)
         }
 
       /* Get item's children and add them to the list */
-      if ((item->has_any_child) && (item->depth < priv->max_depth))
+      if ((item->has_any_child) && (item->depth < priv->max_depth + 1))
         {
           rel_start = 0;
 
@@ -581,16 +594,12 @@ baobab_chart_get_items (GtkWidget *chart, GtkTreePath *root)
 }
 
 static void
-baobab_chart_draw (GtkWidget *chart,
-                   cairo_t *cr,
-                   GdkRectangle area)
+baobab_chart_draw_chart (GtkWidget *chart,
+                         cairo_t *cr)
 {
   BaobabChartPrivate *priv;
   BaobabChartClass *class;
-
   GList *node;
-  BaobabChartItem *item;
-  gboolean highlighted;
 
   priv = BAOBAB_CHART_GET_PRIVATE (chart);
   class = BAOBAB_CHART_GET_CLASS (chart);
@@ -604,10 +613,18 @@ baobab_chart_draw (GtkWidget *chart,
   node = priv->first_item;
   while (node != NULL)
     {
+      BaobabChartItem *item;
+      GdkRectangle clip;
+
       item = (BaobabChartItem *) node->data;
 
-      if ((item->visible) && (gdk_rectangle_intersect (&area, &item->rect, NULL)))
+      if (gdk_cairo_get_clip_rectangle (cr, &clip) &&
+          (item->visible) &&
+          (gdk_rectangle_intersect (&clip, &item->rect, NULL)) &&
+          (item->depth <= priv->max_depth))
         {
+          gboolean highlighted;
+
           highlighted = (node == priv->highlighted_item);
 
           class->draw_item (chart, cr, item, highlighted);
@@ -631,7 +648,7 @@ baobab_chart_update_draw (BaobabChart* chart,
   GtkTreePath *root_path = NULL;
   gint root_depth, node_depth;
 
-  if (!GTK_WIDGET_REALIZED (chart))
+  if (!gtk_widget_get_realized ( GTK_WIDGET (chart)))
     return;
 
   priv = BAOBAB_CHART (chart)->priv;
@@ -737,14 +754,9 @@ baobab_chart_rows_reordered (GtkTreeModel    *model,
 }
 
 static gboolean
-baobab_chart_expose (GtkWidget *chart, GdkEventExpose *event)
+baobab_chart_draw (GtkWidget *chart, cairo_t *cr)
 {
-  cairo_t *cr;
   BaobabChartPrivate *priv;
-  gint w, h;
-  gdouble p, sx, sy;
-  GtkTreePath *root_path = NULL;
-  GtkTreePath *current_path = NULL;
 
   priv = BAOBAB_CHART (chart)->priv;
 
@@ -752,33 +764,27 @@ baobab_chart_expose (GtkWidget *chart, GdkEventExpose *event)
   if (priv->name_column == priv->percentage_column)
     return FALSE;
 
-  /* get a cairo_t */
-  cr = gdk_cairo_create (chart->window);
-
-  cairo_rectangle (cr,
-                   event->area.x, event->area.y,
-                   event->area.width, event->area.height);
-
   /* there is no model we can not paint */
   if ((priv->is_frozen) || (priv->model == NULL))
     {
       if (priv->memento != NULL)
         {
+          gint w, h, aw, ah;
+          gdouble p, sx, sy;
+
           w = cairo_image_surface_get_width (priv->memento);
           h = cairo_image_surface_get_height (priv->memento);
 
-          cairo_clip (cr);
+          aw = gtk_widget_get_allocated_width (chart);
+          ah = gtk_widget_get_allocated_height (chart);
 
-          if (w > 0 && h > 0 &&
-          !(chart->allocation.width == w &&
-                chart->allocation.height == h))
+          if (w > 0 && h > 0 && !(aw == w && aw == h))
             {
               /* minimal available proportion */
-              p = MIN (chart->allocation.width / (1.0 * w),
-                       chart->allocation.height / (1.0 * h));
+              p = MIN (aw / (1.0 * w), ah / (1.0 * h));
 
-              sx = (gdouble) (chart->allocation.width - w * p) / 2.0;
-              sy = (gdouble) (chart->allocation.height - h * p) / 2.0;
+              sx = (gdouble) (aw - w * p) / 2.0;
+              sy = (gdouble) (ah - h * p) / 2.0;
 
               cairo_translate (cr, sx, sy);
               cairo_scale (cr, p, p);
@@ -792,25 +798,30 @@ baobab_chart_expose (GtkWidget *chart, GdkEventExpose *event)
     }
   else
     {
+      GtkTreePath *root_path = NULL;
+
       cairo_set_source_rgb (cr, 1, 1, 1);
       cairo_fill_preserve (cr);
 
-      cairo_clip (cr);
-
       if (priv->root != NULL)
         root_path = gtk_tree_row_reference_get_path (priv->root);
 
-      if (root_path == NULL) {
-        root_path = gtk_tree_path_new_first ();
-        priv->root = NULL;
-      }
+      if (root_path == NULL)
+        {
+          root_path = gtk_tree_path_new_first ();
+          priv->root = NULL;
+        }
 
       /* Check if tree model was modified in any way */
       if ((priv->model_changed) ||
            (priv->first_item == NULL))
-        baobab_chart_get_items (chart, root_path);
+        {
+          baobab_chart_get_items (chart, root_path);
+        }
       else
         {
+          GtkTreePath *current_path;
+
           /* Check if root was changed */
           current_path = gtk_tree_model_get_path (priv->model,
                          &((BaobabChartItem*) priv->first_item->data)->iter);
@@ -823,11 +834,9 @@ baobab_chart_expose (GtkWidget *chart, GdkEventExpose *event)
 
       gtk_tree_path_free (root_path);
 
-      baobab_chart_draw (chart, cr, event->area);
+      baobab_chart_draw_chart (chart, cr);
     }
 
-  cairo_destroy (cr);
-
   return FALSE;
 }
 
@@ -881,17 +890,19 @@ baobab_chart_get_item_color (BaobabChartColor *color,
     }
 
   if (highlighted)
-    if (depth == 0)
-      *color = level_color_hl;
-    else
-      {
-        maximum = MAX (color->red,
-                       MAX (color->green,
-                            color->blue));
-        color->red /= maximum;
-        color->green /= maximum;
-        color->blue /= maximum;
-      }
+    {
+      if (depth == 0)
+        *color = level_color_hl;
+      else
+        {
+          maximum = MAX (color->red,
+                         MAX (color->green,
+                              color->blue));
+          color->red /= maximum;
+          color->green /= maximum;
+          color->blue /= maximum;
+        }
+    }
 }
 
 static gint
@@ -929,20 +940,20 @@ static gint
 baobab_chart_scroll (GtkWidget *widget,
                      GdkEventScroll *event)
 {
-  BaobabChartPrivate * priv = BAOBAB_CHART_GET_PRIVATE (widget);
-
   switch (event->direction)
     {
     case GDK_SCROLL_LEFT :
     case GDK_SCROLL_UP :
-      baobab_chart_zoom_out (widget);
+      if (baobab_chart_can_zoom_out (widget))
+        baobab_chart_zoom_out (widget);
       /* change the selected item when zooming */
       baobab_chart_motion_notify (widget, (GdkEventMotion *)event);
       break;
 
     case GDK_SCROLL_RIGHT :
     case GDK_SCROLL_DOWN :
-      baobab_chart_zoom_in (widget);
+      if (baobab_chart_can_zoom_in (widget))
+        baobab_chart_zoom_in (widget);
       break;
     }
 
@@ -970,7 +981,8 @@ baobab_chart_set_item_highlight (GtkWidget *chart,
   else
     priv->highlighted_item = NULL;
 
-  gdk_window_invalidate_rect (chart->window, &item->rect, TRUE);
+  gdk_window_invalidate_rect (gtk_widget_get_window ( GTK_WIDGET (chart)),
+                              &item->rect, TRUE);
 }
 
 static gint
@@ -1151,13 +1163,11 @@ baobab_chart_get_pixbuf (GtkWidget *widget)
 
   g_return_val_if_fail (BAOBAB_IS_CHART (widget), NULL);
 
-  gdk_drawable_get_size (widget->window, &w, &h);
-  pixbuf = gdk_pixbuf_get_from_drawable (NULL,
-                                         widget->window,
-                                         gdk_colormap_get_system (),
-                                         0, 0,
-                                         0, 0,
-                                         w, h);
+  w = gtk_widget_get_allocated_width (widget);
+  h = gtk_widget_get_allocated_height (widget);
+  pixbuf = gdk_pixbuf_get_from_window (gtk_widget_get_window (widget),
+                                       0, 0,
+                                       w, h);
 
   return pixbuf;
 }
@@ -1385,7 +1395,6 @@ baobab_chart_set_root (GtkWidget *chart,
                        GtkTreePath *root)
 {
   BaobabChartPrivate *priv;
-  GtkTreeIter iter = {0};
   GtkTreePath *current_root;
 
   g_return_if_fail (BAOBAB_IS_CHART (chart));
@@ -1450,8 +1459,7 @@ baobab_chart_freeze_updates (GtkWidget *chart)
 {
   BaobabChartPrivate *priv;
   cairo_surface_t *surface = NULL;
-  cairo_t *cr = NULL;
-  GdkRectangle area;
+  GtkAllocation allocation;
 
   g_return_if_fail (BAOBAB_IS_CHART (chart));
 
@@ -1464,24 +1472,23 @@ baobab_chart_freeze_updates (GtkWidget *chart)
     baobab_chart_disconnect_signals (chart,
                                      priv->model);
 
+  gtk_widget_get_allocation (GTK_WIDGET (chart), &allocation);
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                        chart->allocation.width,
-                                        chart->allocation.height);
+                                        allocation.width,
+                                        allocation.height);
 
   if (cairo_surface_status (surface) == CAIRO_STATUS_SUCCESS)
     {
+      cairo_t *cr;
+
       cr = cairo_create (surface);
 
-      area.x = 0;
-      area.y = 0;
-      area.width = chart->allocation.width;
-      area.height = chart->allocation.height;
-      baobab_chart_draw (chart, cr, area);
+      baobab_chart_draw_chart (chart, cr);
 
       cairo_rectangle (cr,
                        0, 0,
-                       chart->allocation.width,
-                       chart->allocation.height);
+                       allocation.width,
+                       allocation.height);
 
       cairo_set_source_rgba (cr, 0.93, 0.93, 0.93, 0.5);  /* tango: eeeeec */
       cairo_fill_preserve (cr);
@@ -1548,14 +1555,25 @@ baobab_chart_thaw_updates (GtkWidget *chart)
 void
 baobab_chart_zoom_in (GtkWidget *chart)
 {
+  BaobabChartPrivate *priv;
+  BaobabChartClass *class;
+  guint new_max_depth;
+
   g_return_if_fail (BAOBAB_IS_CHART (chart));
 
-  baobab_chart_set_max_depth (chart,
-                              baobab_chart_get_max_depth (chart) - 1);
+  priv = BAOBAB_CHART (chart)->priv;
+  class = BAOBAB_CHART_GET_CLASS (chart);
+
+  if (class->can_zoom_in != NULL)
+    new_max_depth = class->can_zoom_in (chart);
+  else
+    new_max_depth = priv->max_depth - 1;
+
+  baobab_chart_set_max_depth (chart, new_max_depth);
 }
 
 /**
- * baobab_chart_zoom_in:
+ * baobab_chart_zoom_out:
  * @chart: the #BaobabChart requested to zoom out.
  *
  * Zooms out the chart by increasing its maximun depth.
@@ -1696,9 +1714,7 @@ baobab_chart_save_snapshot (GtkWidget *chart)
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fs_dlg),
                                        g_get_home_dir ());
 
-#if GTK_CHECK_VERSION(2,8,0)
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (fs_dlg), TRUE);
-#endif
 
   /* extra widget */
   vbox = gtk_vbox_new (FALSE, 0);
@@ -1714,9 +1730,9 @@ baobab_chart_save_snapshot (GtkWidget *chart)
                       FALSE, FALSE, 0);
 
   opt_menu = gtk_combo_box_new_text ();
-  gtk_combo_box_append_text (GTK_COMBO_BOX (opt_menu), "png");
-  gtk_combo_box_append_text (GTK_COMBO_BOX (opt_menu), "jpeg");
-  gtk_combo_box_append_text (GTK_COMBO_BOX (opt_menu), "bmp");
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (opt_menu), "png");
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (opt_menu), "jpeg");
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (opt_menu), "bmp");
   gtk_combo_box_set_active (GTK_COMBO_BOX (opt_menu), 0);
   gtk_box_pack_start (GTK_BOX (hbox), opt_menu, TRUE, TRUE, 0);
 
@@ -1726,7 +1742,7 @@ baobab_chart_save_snapshot (GtkWidget *chart)
   if (gtk_dialog_run (GTK_DIALOG (fs_dlg)) == GTK_RESPONSE_ACCEPT)
     {
       filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fs_dlg));
-      sel_type = gtk_combo_box_get_active_text (GTK_COMBO_BOX (opt_menu));
+      sel_type = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (opt_menu));
       if (! g_str_has_suffix (filename, sel_type))
         {
           gchar *tmp;
@@ -1784,3 +1800,55 @@ baobab_chart_get_highlighted_item (GtkWidget *chart)
   return (priv->highlighted_item ? 
     (BaobabChartItem *) priv->highlighted_item->data : NULL);
 }
+
+/**
+ * baobab_chart_can_zoom_in:
+ * @chart: the #BaobabChart to ask if can be zoomed in.
+ *
+ * Returns a boolean telling whether the chart can be zoomed in, given its current
+ * visualization conditions.
+ *
+ * Fails if @chart is not a #BaobabChart.
+ **/
+gboolean
+baobab_chart_can_zoom_in (GtkWidget *chart)
+{
+  BaobabChartPrivate *priv;
+  BaobabChartClass *class;
+
+  g_return_val_if_fail (BAOBAB_IS_CHART (chart), FALSE);
+
+  priv = BAOBAB_CHART (chart)->priv;
+  class = BAOBAB_CHART_GET_CLASS (chart);
+
+  if (class->can_zoom_in != NULL)
+    return class->can_zoom_in (chart) > 0;
+  else
+    return priv->max_depth > 1;
+}
+
+/**
+ * baobab_chart_can_zoom_out:
+ * @chart: the #BaobabChart to ask if can be zoomed out.
+ *
+ * Returns a boolean telling whether the chart can be zoomed out, given its current
+ * visualization conditions.
+ *
+ * Fails if @chart is not a #BaobabChart.
+ **/
+gboolean
+baobab_chart_can_zoom_out (GtkWidget *chart)
+{
+  BaobabChartPrivate *priv;
+  BaobabChartClass *class;
+
+  g_return_val_if_fail (BAOBAB_IS_CHART (chart), FALSE);
+
+  priv = BAOBAB_CHART (chart)->priv;
+  class = BAOBAB_CHART_GET_CLASS (chart);
+
+  if (class->can_zoom_out != NULL)
+    return class->can_zoom_out (chart) > 0;
+  else
+    return (priv->max_depth < BAOBAB_CHART_MAX_DEPTH);
+}
