@@ -39,9 +39,8 @@
 #include "baobab-utils.h"
 #include "callbacks.h"
 
-
 void
-baobab_get_filesystem (baobab_fs *fs)
+baobab_get_filesystem (BaobabFS *fs)
 {
 	size_t i;
 	glibtop_mountlist mountlist;
@@ -52,24 +51,21 @@ baobab_get_filesystem (baobab_fs *fs)
 	mountentries = glibtop_get_mountlist (&mountlist, FALSE);
 
 	for (i = 0; i < mountlist.number; ++i) {
-		GFile	*file;
+		GFile *file;
 		glibtop_fsusage fsusage;
-		
-		file = g_file_new_for_path(mountentries[i].mountdir);	
 
-		if (baobab_is_excluded_location (file)){
-			g_object_unref(file);
-			continue;
-			}
+		file = g_file_new_for_path (mountentries[i].mountdir);
 
-		glibtop_get_fsusage (&fsusage, mountentries[i].mountdir);
+		if (!baobab_is_excluded_location (file)) {
 
-		/*  v.1.1.1 changed bavail with bfree) */
-		fs->total += fsusage.blocks * fsusage.block_size;
-		fs->avail += fsusage.bfree * fsusage.block_size;
-		fs->used += (fsusage.blocks - fsusage.bfree) * fsusage.block_size;
-		g_object_unref(file);
+			glibtop_get_fsusage (&fsusage, mountentries[i].mountdir);
 
+			fs->total += fsusage.blocks * fsusage.block_size;
+			fs->avail += fsusage.bfree * fsusage.block_size;
+			fs->used += (fsusage.blocks - fsusage.bfree) * fsusage.block_size;
+		}
+
+		g_object_unref (file);
 	}
 
 	g_free (mountentries);
@@ -279,7 +275,7 @@ messageyesno (const gchar *primary_msg,
 	                                          "%s", secondary_msg);
 
 	button = gtk_button_new_with_mnemonic (ok_button);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 	gtk_widget_show (button);
 	
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
@@ -356,9 +352,8 @@ add_popupmenu_item (GtkMenu *pmenu, const gchar *label, const gchar *stock, GCal
 void
 popupmenu_list (GtkTreePath *path, GdkEventButton *event, gboolean can_trash)
 {
-	GtkWidget *pmenu, *open, *trash, *remove;
+	GtkWidget *pmenu;
 	gchar *path_to_string;
-	GtkWidget *image;
 
 	/* path_to_string is freed in callback function */
 	path_to_string = gtk_tree_path_to_string (path);
@@ -382,44 +377,6 @@ popupmenu_list (GtkTreePath *path, GdkEventButton *event, gboolean can_trash)
 			event->button, event->time);
 }
 
-void
-set_label_scan (baobab_fs *fs)
-{
-	gchar *markup;
-	gchar *total;
-	gchar *used;
-	gchar *available;
-
-	g_free (baobab.label_scan);
-
-	total = g_format_size_for_display (fs->total);
-	used = g_format_size_for_display (fs->used);
-	available = g_format_size_for_display (fs->avail);
-
-	/* Translators: these are labels for disk space */
-	markup = g_markup_printf_escaped  ("<small>%s <b>%s</b> (%s %s %s %s )</small>",
-					   _("Total filesystem capacity:"), total,
-					   _("used:"), used,
-					   _("available:"), available);
-
-	baobab.label_scan = markup;
-
-	g_free (total);
-	g_free (used);
-	g_free (available);
-}
-
-void
-show_label (void)
-{
-	GtkWidget *label;
-
-	label = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "label1"));
-
-	gtk_label_set_markup (GTK_LABEL (label),
-			      baobab.label_scan);
-}
-
 void
 open_file_with_application (GFile *file)
 {
@@ -538,17 +495,6 @@ trash_file (GFile *file)
 	return TRUE;
 }
 
-gchar *
-baobab_gconf_get_string_with_default (GConfClient *client,
-				      const gchar *key,
-				      const gchar *def)
-{
-	gchar *val;
-
-	val = gconf_client_get_string (client, key, NULL);
-	return val ? val : g_strdup (def);
-}
-
 gboolean
 baobab_help_display (GtkWindow   *parent,
 		     const gchar *file_name,
