@@ -210,7 +210,6 @@ static gint n_toggle_state = G_N_ELEMENTS (toggle_state);
 static void
 gdict_window_ensure_menu_state (GdictWindow *window)
 {
-  GtkWidget *item;
   gint i;
   gboolean is_sensitive;
 
@@ -325,7 +324,7 @@ gdict_window_lookup_start_cb (GdictContext *context,
   window->last_definition = 0;
   window->current_definition = 0;
 
-  gdk_window_set_cursor (GTK_WIDGET (window)->window, window->busy_cursor);
+  gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), window->busy_cursor);
 
   g_free (message);
 }
@@ -381,7 +380,7 @@ gdict_window_lookup_end_cb (GdictContext *context,
                           -1);
     }
 
-  gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+  gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
   g_free (message);
 
   if (count == 0)
@@ -398,9 +397,7 @@ gdict_window_error_cb (GdictContext *context,
 		       const GError *error,
 		       GdictWindow  *window)
 {
-  gint count;
-  
-  gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+  gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
   
   if (window->status && window->statusbar_visible)
     gtk_statusbar_push (GTK_STATUSBAR (window->status), 0,
@@ -1006,8 +1003,6 @@ static void
 gdict_window_cmd_view_sidebar (GtkAction   *action,
 			       GdictWindow *window)
 {
-  gboolean is_active;
-  
   g_assert (GDICT_IS_WINDOW (window));
 
   if (window->sidebar_visible)
@@ -1553,8 +1548,6 @@ gdict_window_drag_data_received_cb (GtkWidget        *widget,
   text = (gchar *) gtk_selection_data_get_text (data);
   if (text)
     {
-      gchar *title;
-      
       gtk_entry_set_text (GTK_ENTRY (window->entry), text);
 
       gdict_window_set_word (window, text, NULL);
@@ -1625,7 +1618,7 @@ set_window_default_size (GdictWindow *window)
       /* Size based on the font size */
       GtkWidget *defbox = window->defbox;
       
-      font_size = pango_font_description_get_size (defbox->style->font_desc);
+      font_size = pango_font_description_get_size (gtk_widget_get_style (defbox)->font_desc);
       font_size = PANGO_PIXELS (font_size);
 
       width = font_size * GDICT_WINDOW_COLUMNS;
@@ -1638,7 +1631,7 @@ set_window_default_size (GdictWindow *window)
 
       /* ... but make it no larger than the monitor */
       screen = gtk_widget_get_screen (widget);
-      monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+      monitor_num = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget));
 
       gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
       width = MIN (width, monitor.width * 3 / 4);
@@ -1658,7 +1651,6 @@ static void
 gdict_window_style_set (GtkWidget *widget,
 			GtkStyle  *old_style)
 {
-  GdictWindow *window;
 
   if (GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set)
     GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set (widget, old_style);
@@ -1673,9 +1665,11 @@ gdict_window_handle_notify_position_cb (GtkWidget  *widget,
 {
   GdictWindow *window = GDICT_WINDOW (user_data);
   gint window_width, pos;
+  GtkAllocation allocation;
 
   pos = gtk_paned_get_position (GTK_PANED (widget));
-  window_width = GTK_WIDGET (window)->allocation.width;
+  gtk_widget_get_allocation (GTK_WIDGET (window), &allocation);
+  window_width = allocation.width;
 
   window->sidebar_width = window_width - pos;
 }
@@ -1696,12 +1690,12 @@ gdict_window_constructor (GType                  type,
   GtkWidget *button;
   GtkActionGroup *action_group;
   GtkAccelGroup *accel_group;
-  GtkAction *action;
   PangoFontDescription *font_desc;
   gchar *font_name, *sidebar_page;
   GError *error;
   gboolean sidebar_visible;
   gboolean statusbar_visible;
+  GtkAllocation allocation;
   
   object = G_OBJECT_CLASS (gdict_window_parent_class)->constructor (type,
   						   n_construct_properties,
@@ -1885,7 +1879,6 @@ gdict_window_constructor (GType                  type,
   gtk_widget_show (window->defbox_frame);
 
   window->status = gtk_statusbar_new ();
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->status), TRUE);
   gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
   statusbar_visible = gconf_client_get_bool (window->gconf_client,
 		  			     GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
@@ -1957,8 +1950,8 @@ gdict_window_constructor (GType                  type,
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (window));
 
-  gtk_paned_set_position (GTK_PANED (handle),
-		          GTK_WIDGET (window)->allocation.width - sidebar_width);
+  gtk_widget_get_allocation (GTK_WIDGET (window), &allocation);
+  gtk_paned_set_position (GTK_PANED (handle), allocation.width - sidebar_width);
   if (sidebar_page)
     {
       gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), sidebar_page);
