@@ -91,7 +91,6 @@ gboolean
 screenshot_grab_lock (void)
 {
   GdkAtom selection_atom;
-  GdkCursor *cursor;
   gboolean result = FALSE;
 
   selection_atom = gdk_atom_intern (SELECTION_NAME, FALSE);
@@ -267,18 +266,59 @@ select_area_button_press (XKeyEvent    *event,
   draw_rect->height = 0;
 }
 
+static void
+select_area_update_rect (GtkWidget    *window,
+                         GdkRectangle *rect)
+{
+  if (rect->width <= 0 || rect->height <= 0)
+    {
+      gtk_widget_hide (window);
+      return;
+    }
+
+  gtk_window_move (GTK_WINDOW (window), rect->x, rect->y);
+  gtk_window_resize (GTK_WINDOW (window), rect->width, rect->height);
+  gtk_widget_show (window);
+  
+  /* We (ab)use app-paintable to indicate if we have an RGBA window */
+  if (!gtk_widget_get_app_paintable (window))
+    {
+      GdkWindow *gdkwindow = gtk_widget_get_window (window);
+
+      /* Shape the window to make only the outline visible */
+      if (rect->width > 2 && rect->height > 2)
+        {
+          cairo_region_t *region, *region2;
+          cairo_rectangle_int_t region_rect = {
+            0, 0,
+            rect->width - 2, rect->height - 2
+          };
+
+          region = cairo_region_create_rectangle (&region_rect);
+          region_rect.x++;
+          region_rect.y++;
+          region_rect.width -= 2;
+          region_rect.height -= 2;
+          region2 = cairo_region_create_rectangle (&region_rect);
+          cairo_region_subtract (region, region2);
+
+          gdk_window_shape_combine_region (gdkwindow, region, 0, 0);
+
+          cairo_region_destroy (region);
+          cairo_region_destroy (region2);
+        }
+      else
+        gdk_window_shape_combine_region (gdkwindow, NULL, 0, 0);
+    }
+}
+
 static void
 select_area_button_release (XKeyEvent    *event,
                             GdkRectangle *rect,
                             GdkRectangle *draw_rect,
-                            GdkWindow    *root,
-                            GdkGC        *gc)
+                            GtkWidget    *window)
 {
-  /* remove the old rectangle */
-  if (draw_rect->width > 0 && draw_rect->height > 0)
-    gdk_draw_rectangle (root, gc, FALSE, 
-                        draw_rect->x, draw_rect->y,
-                        draw_rect->width, draw_rect->height);
+  gtk_widget_hide (window);
 
   rect->width  = ABS (rect->x - event->x_root);
   rect->height = ABS (rect->y - event->y_root);
@@ -291,39 +331,82 @@ static void
 select_area_motion_notify (XKeyEvent    *event,
                            GdkRectangle *rect,
                            GdkRectangle *draw_rect,
-                           GdkWindow    *root,
-                           GdkGC        *gc)
+                           GtkWidget    *window)
 {
-  /* FIXME: draw some nice rubberband with cairo if composited */
-
-  /* remove the old rectangle */
-  if (draw_rect->width > 0 && draw_rect->height > 0)
-    gdk_draw_rectangle (root, gc, FALSE, 
-                        draw_rect->x, draw_rect->y,
-                        draw_rect->width, draw_rect->height);
-
   draw_rect->width  = ABS (rect->x - event->x_root);
   draw_rect->height = ABS (rect->y - event->y_root);
 
   draw_rect->x = MIN (rect->x, event->x_root);
   draw_rect->y = MIN (rect->y, event->y_root);
 
-  /* draw the new rectangle */
-  if (draw_rect->width > 0 && draw_rect->height > 0)
-    gdk_draw_rectangle (root, gc, FALSE, 
-                        draw_rect->x, draw_rect->y,
-                        draw_rect->width, draw_rect->height);
+  select_area_update_rect (window, draw_rect);
 }
 
 typedef struct {
   GdkRectangle  rect;
   GdkRectangle  draw_rect;
   gboolean      button_pressed;
-  /* only needed because we're not using cairo to draw the rectangle */
-  GdkWindow    *root;
-  GdkGC        *gc;
+  GtkWidget    *window;
 } select_area_filter_data;
 
+static gboolean
+select_window_draw (GtkWidget *window, cairo_t *cr, gpointer unused)
+{
+  GtkAllocation allocation;
+  GtkStyle *style;
+
+  style = gtk_widget_get_style (window);
+
+  if (gtk_widget_get_app_paintable (window))
+    {
+      cairo_set_line_width (cr, 1.0);
+
+      gtk_widget_get_allocation (window, &allocation);
+
+      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+      cairo_set_source_rgba (cr, 0, 0, 0, 0);
+      cairo_paint (cr);
+
+      cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+      gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_SELECTED]);
+      cairo_paint_with_alpha (cr, 0.25);
+
+      cairo_rectangle (cr, 
+                       allocation.x + 0.5, allocation.y + 0.5,
+                       allocation.width - 1, allocation.height - 1);
+      cairo_stroke (cr);
+    }
+  else
+    {
+      gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_SELECTED]);
+      cairo_paint (cr);
+    }
+
+  return TRUE;
+}
+
+static GtkWidget *
+create_select_window (void)
+{
+  GtkWidget *window;
+  GdkScreen *screen;
+  GdkVisual *visual;
+
+  screen = gdk_screen_get_default ();
+  visual = gdk_screen_get_rgba_visual (screen);
+
+  window = gtk_window_new (GTK_WINDOW_POPUP);
+  if (gdk_screen_is_composited (screen) && visual)
+    {
+      gtk_widget_set_visual (window, visual);
+      gtk_widget_set_app_paintable (window, TRUE);
+    }
+
+  g_signal_connect (window, "draw", G_CALLBACK (select_window_draw), NULL);
+
+  return window;
+}
+
 static GdkFilterReturn
 select_area_filter (GdkXEvent *gdk_xevent,
                     GdkEvent  *event,
@@ -347,7 +430,7 @@ select_area_filter (GdkXEvent *gdk_xevent,
       {
         select_area_button_release (&xevent->xkey,
                                     &data->rect, &data->draw_rect,
-                                    data->root, data->gc);
+                                    data->window);
         gtk_main_quit ();
       }
       return GDK_FILTER_REMOVE;
@@ -355,10 +438,10 @@ select_area_filter (GdkXEvent *gdk_xevent,
       if (data->button_pressed)
         select_area_motion_notify (&xevent->xkey,
                                    &data->rect, &data->draw_rect,
-                                   data->root, data->gc);
+                                   data->window);
       return GDK_FILTER_REMOVE;
     case KeyPress:
-      if (xevent->xkey.keycode == XKeysymToKeycode (gdk_display, XK_Escape))
+      if (xevent->xkey.keycode == XKeysymToKeycode (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XK_Escape))
         {
           data->rect.x = 0;
           data->rect.y = 0;
@@ -384,8 +467,6 @@ screenshot_select_area (int *px,
   GdkWindow               *root;
   GdkCursor               *cursor;
   select_area_filter_data  data;
-  GdkGCValues              values;
-  GdkColor                 color;
 
   root = gdk_get_default_root_window ();
   cursor = gdk_cursor_new (GDK_CROSSHAIR);
@@ -415,38 +496,14 @@ screenshot_select_area (int *px,
   data.rect.width  = 0;
   data.rect.height = 0;
   data.button_pressed = FALSE;
-  data.root = root;
-
-  values.function = GDK_XOR;
-  values.fill = GDK_SOLID;
-  values.clip_mask = NULL;
-  values.subwindow_mode = GDK_INCLUDE_INFERIORS;
-  values.clip_x_origin = 0;
-  values.clip_y_origin = 0;
-  values.graphics_exposures = 0;
-  values.line_width = 0;
-  values.line_style = GDK_LINE_SOLID;
-  values.cap_style = GDK_CAP_BUTT;
-  values.join_style = GDK_JOIN_MITER;
-
-  data.gc = gdk_gc_new_with_values (root, &values,
-                                    GDK_GC_FUNCTION | GDK_GC_FILL |
-                                    GDK_GC_CLIP_MASK | GDK_GC_SUBWINDOW |
-                                    GDK_GC_CLIP_X_ORIGIN |
-                                    GDK_GC_CLIP_Y_ORIGIN | GDK_GC_EXPOSURES |
-                                    GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE |
-                                    GDK_GC_CAP_STYLE | GDK_GC_JOIN_STYLE);
-  gdk_color_parse ("white", &color);
-  gdk_gc_set_rgb_fg_color (data.gc, &color);
-  gdk_color_parse ("black", &color);
-  gdk_gc_set_rgb_bg_color (data.gc, &color);
+  data.window = create_select_window();
 
   gtk_main ();
 
-  g_object_unref (data.gc);
-
   gdk_window_remove_filter (root, (GdkFilterFunc) select_area_filter, &data);
 
+  gtk_widget_destroy (data.window);
+
   gdk_keyboard_ungrab (GDK_CURRENT_TIME);
   gdk_pointer_ungrab (GDK_CURRENT_TIME);
   gdk_cursor_unref (cursor);
@@ -467,8 +524,8 @@ find_wm_window (Window xid)
 
   do
     {
-      if (XQueryTree (GDK_DISPLAY (), xid, &root,
-		      &parent, &children, &nchildren) == 0)
+      if (XQueryTree (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                      xid, &root, &parent, &children, &nchildren) == 0)
 	{
 	  g_warning ("Couldn't find window manager window");
 	  return None;
@@ -482,23 +539,23 @@ find_wm_window (Window xid)
   while (TRUE);
 }
 
-static GdkRegion *
+static cairo_region_t *
 make_region_with_monitors (GdkScreen *screen)
 {
-  GdkRegion *region;
+  cairo_region_t *region;
   int num_monitors;
   int i;
 
   num_monitors = gdk_screen_get_n_monitors (screen);
 
-  region = gdk_region_new ();
+  region = cairo_region_create ();
 
   for (i = 0; i < num_monitors; i++)
     {
       GdkRectangle rect;
 
       gdk_screen_get_monitor_geometry (screen, i, &rect);
-      gdk_region_union_with_rect (region, &rect);
+      cairo_region_union_rectangle (region, &rect);
     }
 
   return region;
@@ -545,15 +602,14 @@ blank_rectangle_in_pixbuf (GdkPixbuf *pixbuf, GdkRectangle *rect)
 }
 
 static void
-blank_region_in_pixbuf (GdkPixbuf *pixbuf, GdkRegion *region)
+blank_region_in_pixbuf (GdkPixbuf *pixbuf, cairo_region_t *region)
 {
-  GdkRectangle *rects;
   int n_rects;
   int i;
   int width, height;
-  GdkRectangle pixbuf_rect;
+  cairo_rectangle_int_t pixbuf_rect;
 
-  gdk_region_get_rectangles (region, &rects, &n_rects);
+  n_rects = cairo_region_num_rectangles (region);
 
   width = gdk_pixbuf_get_width (pixbuf);
   height = gdk_pixbuf_get_height (pixbuf);
@@ -565,13 +621,12 @@ blank_region_in_pixbuf (GdkPixbuf *pixbuf, GdkRegion *region)
 
   for (i = 0; i < n_rects; i++)
     {
-      GdkRectangle dest;
+      cairo_rectangle_int_t rect, dest;
 
-      if (gdk_rectangle_intersect (rects + i, &pixbuf_rect, &dest))
+      cairo_region_get_rectangle (region, i, &rect);
+      if (gdk_rectangle_intersect (&rect, &pixbuf_rect, &dest))
 	blank_rectangle_in_pixbuf (pixbuf, &dest);
     }
-
-  g_free (rects);
 }
 
 /* When there are multiple monitors with different resolutions, the visible area
@@ -584,11 +639,11 @@ static void
 mask_monitors (GdkPixbuf *pixbuf, GdkWindow *root_window)
 {
   GdkScreen *screen;
-  GdkRegion *region_with_monitors;
-  GdkRegion *invisible_region;
-  GdkRectangle rect;
+  cairo_region_t *region_with_monitors;
+  cairo_region_t *invisible_region;
+  cairo_rectangle_int_t rect;
 
-  screen = gdk_drawable_get_screen (GDK_DRAWABLE (root_window));
+  screen = gdk_window_get_screen (root_window);
 
   region_with_monitors = make_region_with_monitors (screen);
 
@@ -597,13 +652,13 @@ mask_monitors (GdkPixbuf *pixbuf, GdkWindow *root_window)
   rect.width = gdk_screen_get_width (screen);
   rect.height = gdk_screen_get_height (screen);
 
-  invisible_region = gdk_region_rectangle (&rect);
-  gdk_region_subtract (invisible_region, region_with_monitors);
+  invisible_region = cairo_region_create_rectangle (&rect);
+  cairo_region_subtract (invisible_region, region_with_monitors);
 
   blank_region_in_pixbuf (pixbuf, invisible_region);
 
-  gdk_region_destroy (region_with_monitors);
-  gdk_region_destroy (invisible_region);
+  cairo_region_destroy (region_with_monitors);
+  cairo_region_destroy (invisible_region);
 }
 
 GdkPixbuf *
@@ -634,7 +689,8 @@ screenshot_get_pixbuf (GdkWindow    *window,
 
   root = gdk_get_default_root_window ();
 
-  gdk_drawable_get_size (window, &real_width, &real_height);      
+  real_width = gdk_window_get_width (window);
+  real_height = gdk_window_get_height (window);
   gdk_window_get_origin (window, &x_real_orig, &y_real_orig);
 
   x_orig = x_real_orig;
@@ -667,10 +723,10 @@ screenshot_get_pixbuf (GdkWindow    *window,
       width  = rectangle->width;
       height = rectangle->height;
     }
-  
-  screenshot = gdk_pixbuf_get_from_drawable (NULL, root, NULL,
-                                             x_orig, y_orig, 0, 0,
-                                             width, height);
+
+  screenshot = gdk_pixbuf_get_from_window (root,
+                                           x_orig, y_orig,
+                                           width, height);
 
   mask_monitors (screenshot, root);
 
@@ -685,7 +741,7 @@ screenshot_get_pixbuf (GdkWindow    *window,
        * of the WM decoration.
        */
 
-      rectangles = XShapeGetRectangles (GDK_DISPLAY (),
+      rectangles = XShapeGetRectangles (GDK_DISPLAY_XDISPLAY (gdk_display_get_default()),
                                         GDK_WINDOW_XWINDOW (window),
                                         ShapeBounding,
                                         &rectangle_count,
@@ -834,7 +890,8 @@ screenshot_show_error_dialog (GtkWindow   *parent,
                               const gchar *detail)
 {
   GtkWidget *dialog;
-  
+  GtkWindowGroup *group;
+
   g_return_if_fail ((parent == NULL) || (GTK_IS_WINDOW (parent)));
   g_return_if_fail (message != NULL);
   
@@ -848,10 +905,14 @@ screenshot_show_error_dialog (GtkWindow   *parent,
   if (detail)
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
   					      "%s", detail);
-  
-  if (parent && parent->group)
-    gtk_window_group_add_window (parent->group, GTK_WINDOW (dialog));
-  
+
+  if (parent)
+    {
+      group = gtk_window_get_group (parent);
+      if (group != NULL)
+        gtk_window_group_add_window (group, GTK_WINDOW (dialog));
+    }
+
   gtk_dialog_run (GTK_DIALOG (dialog));
   
   gtk_widget_destroy (dialog);
