@@ -55,7 +55,6 @@ static GOptionContext *
 create_option_context (void)
 {
   GOptionContext *context;
-  GOptionGroup *group;
 
   const GOptionEntry entries[] = {
     { "version", 'V', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
@@ -66,6 +65,7 @@ create_option_context (void)
   };
 
   context = g_option_context_new (_(" - Browse and monitor logs"));
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
   g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
   g_option_context_set_ignore_unknown_options (context, TRUE);
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
